# 🚀 programmering.no | 🤓 matematikk.as
# - Ser om et element eksisterer i en liste

# Verdier
liste = ["elefant", "ulv", "rev"]
dyr   = "elefant"

# Print liste
print(f"Liste: {liste}")

# Itererer gjennom listen
for el in liste:

    # El eksisterer i listen
    if el == dyr:

        # Print el
        print(f"Ja, {dyr} eksisterer i listen")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 2) - Sparebeløp
# - Løser oppgaven med CAS i Python
# - Kopier denne kommandoen i terminalen for å importere matematikk:
#   $ pip install matematikk
# - Klasser og funksjoner som også er i matematikk
#   - from os import system     -> os.system(<posix-command>)   -> Eks: os.system("clear")
#   - from copy import deepcopy -> copy.deepcopy(<objekt>)      -> Eks: mat_2 = copy.deepcopy(mat_1)
#   - from numpy import random  -> random.randint(<int>))       -> Eks: random.randint(len(matrix_jp_ls))
#   - import colorama           -> ...                          -> Eks: colorama.Fore.BLUE

import matematikk as mt


# Tøm terminal-skjerm for hvert run
_blokk = 1
if _blokk == 1:

    # Fungerer perfekt, men printer [H[2J[3J i editoren på programmering.no pga. en JavaScript-bug
    mt.system("clear")

# Variabler
if _blokk == 1:

    # Tabell
    _Periode_str        = "Periode"         # Periode n: Brukes flere ganger og er derfor en variabel (unngår skrifeil i strings)
    _Aar_str            = "År"              # År (YYYY): -"-
    _Innskudd_str       = "Innskudd"        # Inskudd: -"-"
    _Innskudd_d_str     = "Innskudd d"      # Inskudd d: Fast øking av innskudd (differansen i aritmetisk rekke)
    _B_start_str        = "B (start)"       # Beløp (start): Beløpet på konto i starten av perioden (året)
    _B_slutt_str        = "B (slutt)"       # Beløp (slutt): Beløpet på konto i slutten av perioden (året)
    tab_mat_0           = [                 # Miriam: Matrise-header for tabell-verdier (kopier med deepcopy for ikke å skrive over)
        [_Periode_str],                     #
        # [_B_start_str],                   # B (start): Tabellen kan vise kolonnene i valg-fri rekkefølge
        [_Aar_str],                         #
        [_Innskudd_str],                    # Innskudd
        [_Innskudd_d_str],                  # Innskudd d
        [_B_start_str],                     # Beløp (start)
        [_B_slutt_str]                      # Beløp (slutt)
    ]
    tab_bredde          = 12                # Antall char bredde på tabell-kolonne
    tab_print_1_start   = 0                 # Start-verdi for å printe tabell
    tab_print_1_slutt   = 2                 # Slutt-verdi for å printe tabell
    tab_print_2_start   = 19                # Start-verdi for å printe tabell
    tab_print_2_slutt   = 20                # Slutt-verdi for å printe tabell
    tab_desimal         = None              # Miriam: Avrunding svar i a)
    tab_is_farge        = 1                 # Hermod: Matrix effekt
    tab_farge_typ_mat   = "grønn"           # Hermod: Matrix-farge (grønn!!!)
    tab_farge_typ_svar  = "blå"             # Hermod: Svar-setninger

    # Spinner
    is_spinning         = 0                 # Vente-spinner når while-løkken kjører, kan ta lang tid, avbryt med ctrl + z

    # Matrix-effekt
    if _blokk == 1:
        matrix_jp_ls        = [             # Liste med japanske symbol (jp-sym) som hentes frem tilfeldig (alleer dobbelt så lange som vanlige char)
            "グ", "ケ", "ゲ", "コ", "ゴ", "サ", "ザ", "シ", "ジ", "ス", "ズ", "セ", "ゼ", "ソ", "ゾ", "タ"
        ]
        matrix_null_en_ls   = ["0", "1"]    # Liste med 0 og 1 som blandes med jp-sym
        matrix_f_ls         = [             # Filter-parametre som gir en tilfeldig, dynamisk blanding mellom to lister (se nedenfor): 3 i dette scopet og 4 som argumenter inn i tabell
            1.00, 0.10, 0.60, 0.80, 0.99, 0.10, 0.99
        ]
        matrix_cut_off      = 700000 / 10   # Gjør at flere el kommer etter hverandre (startet med 692 852 som er ca. 700 000 / 140 = 5000 el etter hverandre)

    # Oppg a)
    innskudd_0_mi       = 20000             # Miriam: Årlig innskudd (starten av året)
    innskudd_mi         = innskudd_0_mi     # Miriam: Årlig innskudd (starten av året), kopiers så slipper å resette før f.eks. CAS
    rente_mi            = 3.5               # Miriam: Rente
    vf_mi               = mt.vekstfaktor(   # Miriam: Vekstfaktor
        fortegn         = "+",
        p               = rente_mi,
        desimal         = -1
    )
    n_mi                = 20                # Miriam: Spare-periode, f.eks. 20.0 gir type-advarsel
    aar_start_mi        = 2024              # Miriam: Sparing start, f.eks. 2024.0 gir type-advarsel
    b_start_mi          = float()                   # Miriam: Start-beløp på konto etter n år
    b_slutt_mi          = float()                   # Miriam: Slutt-beløp på konto etter n år
    INSK_mi             = mt.Symbol("INSK_mi")      # Miriam: CAS-variabel for årlig innskudd
    b_n_mi_1            = float()                   # Miriam: CAS-variabel for beløp nr. n
    b_sum_mi_1          = 0                         # Miriam: CAS-variabel for summen av beløper
    b_20_mi_1           = 565594                    # Miriam: CAS: Beløps-grense like etter innskudd nr. 20

    # Oppg b)
    rente_he            = 3.5                       # Hermod: Rente
    vf_he               = mt.vekstfaktor(           # Hermod: Vekstfaktor
        "+",
        rente_he
    )
    n_he                = 20                        # Hermod: Spare-periode, f.eks. 20.0 gir type-advarsel
    aar_start_he        = 2024                      # Hermod: Sparing start, f.eks. 2024.0 gir type-advarsel
    INSK_he             = mt.Symbol("INSK_HE")     # Hermod: CAS-variabel for årlig innskudd
    b_20_he             = 692852                    # Hermod: Beløps-grense like etter innskudd nr. 20
    B_n_he              = mt.Symbol("B_n_he")       # Hermod: CAS-variabel for beløp nr. n
    B_sum_he            = 0                         # Hermod: CAS-variabel for summen av beløper
    belop_debug         = 0                         # Hermod: Debug utregningene av start-beløp i while-løkken (tar lang tid)

    # Oppg c)
    INSK_d_mi           = mt.Symbol("INSK_d_mi")    # Miriam: CAS-variabel for fast øking av innskudd
    B_n_mi_2            = mt.Symbol("B_n_mi_2")     # Miriam: CAS-variabel for beløp nr. n
    B_sum_mi_2          = 0                         # Miriam: CAS-variabel for summen av beløper
    b_20_mi_2           = 1000000                   # Miriam: Beløps-grense like etter innskudd nr. 20

# a) Miriam > Vis at Miriam har 565 594 kr like etter innskudd 20 > Formel
if _blokk == 1:

    # Variabler
    _n_periode = mt.sjekk_datatype(vari_navn = "_n_periode", vari = n_mi, typ = int)

    # Geometrisk formel: b_n = b_1 * k ** (n - 1)
    for n in range(1, _n_periode + 1):
        b_n_mi_1    = (innskudd_0_mi) * vf_mi ** (n - 1)
        b_sum_mi_1  += b_n_mi_1

    # Avrunding
    b_sum_mi_1 = round(b_sum_mi_1, tab_desimal)

    # Print svar-setninger
    mt.oppg_strek(1)
    print(f"")
    print(f"- Miriam har {b_sum_mi_1} kr like etter innskudd {n_mi}, QED")

# a) Miriam > Vis at Miriam har 565 594 kr like etter innskudd 20 > Tabell (rekker)
if _blokk == -1:

    mt.oppg_strek(1)
    print(f""); print(f"Oppg a) Miriam: Vis beløp 565 594 kr etter innskudd {n_mi}"); print(f"")

    # Debug
    _debug_mat = 0

    # Resetting
    tab_mat = mt.deepcopy(tab_mat_0)

    # Regner og printer ut tabell
    tab_mat = mt.tabell_økonomi_lag(
        tab_mat             = tab_mat,
        tab_desimal         = None,                 # None, -1
        tab_print           = 1,
        tab_print_1_start   = tab_print_1_start,    # tab_print_1_start,
        tab_print_1_slutt   = tab_print_1_slutt,    # tab_print_1_slutt, n_mi
        tab_print_2_start   = tab_print_2_start,    # tab_print_2_start, 0
        tab_print_2_slutt   = tab_print_2_slutt,    # tab_print_2_slutt, 0
        tab_bredde          = tab_bredde,
        vf                  = vf_mi,
        n_periode           = n_mi,
        aar_start           = aar_start_mi,
        innskudd            = innskudd_mi,
        innskudd_desimal    = -1,                   # -1, None,
        innskudd_d          = 0,
        innskudd_d_desimal  = -1,                   # -1, None
        b_start_desimal     = -1,                   # -1, None
        b_slutt_desimal     = -1,                   # -1, None
    )

    # Debug
    if _debug_mat == 1:
        print(""); print("debug > done ::")
        for j in range(len(tab_mat)): print(tab_mat[j])

    # Finn start-beløpet
    tab_mat_el = mt.tabell_finn_el(tab_mat, n_mi, _B_start_str)

    # Print svar-setninger
    print(f"")
    print(f"- Miriam har {tab_mat_el} kr like etter innskudd {n_mi}, QED")

# b) Hermod > Manuell brute force med binært søk
if _blokk == -1:

    # Resetting
    tab_mat = mt.deepcopy(tab_mat_0)

    # 692852: Prøver verdier fra 0 til 1836.33
    innskudd_he = 24500

    # Lager tabell
    tab_mat = mt.tabell_økonomi_lag(
        tab_mat             = tab_mat,
        tab_desimal         = None,                 # None, -1
        tab_print           = -1,
        tab_bredde          = tab_bredde,
        vf                  = vf_he,
        n_periode           = n_he,
        aar_start           = aar_start_he,
        innskudd            = innskudd_he,
        innskudd_desimal    = 2,                   # -1, None,
        innskudd_d          = 0,
        innskudd_d_desimal  = -1,                   # -1, None
        b_start_desimal     = -1,                   # -1, None
        b_slutt_desimal     = -1,                   # -1, None
    )

    # Finn start-beløpet
    tab_mat_el = mt.tabell_finn_el(tab_mat, n_he, _B_start_str)

    # Print tabell og svar-setninger
    mt.oppg_strek(1)
    print(f""); print(f"Oppg b) Hermod: Manuell brute force med binært søk"); print("")
    mt.tabell_print( # mt
        tab_mat             = tab_mat,
        tab_print_1_start   = tab_print_1_start,
        tab_print_1_slutt   = tab_print_1_slutt,
        tab_print_2_start   = tab_print_2_start,
        tab_print_2_slutt   = tab_print_2_slutt,
        tab_bredde          = tab_bredde,
        n_periode           = n_he
    )
    print(f"")
    print(f"- Hermod må ha et årlig innskudd på ca. {innskudd_he} kr")
    print(f"  for å få {tab_mat_el} kr like etter innskudd {n_he} ")

# b) Hermod > While-løkke - Matrix-effekt
if _blokk == -1:

    # While-løkken kjører så lenge beløpet er mindre enn 1 000 000
    _n_periode          = mt.sjekk_datatype(vari_navn = "n_periode", vari = n_he, typ = int)
    innskudd_he         = 0 # Debug fra 24000
    belop_start_ink_he  = 0 # Start på 0

    # Spinner
    i_spin              = 0 # Telle-variabel for spinner

    # Videre som argument inn i tabell
    _mat_matrix = mt.tabell_print_effekt(n_perioder    = b_20_he,
                                      mat_fx        = ["matrix", matrix_jp_ls, matrix_null_en_ls, matrix_f_ls, matrix_cut_off])

    # Kjør løkken så lenge start-beløpet nr. 20 er mindre enn en mill
    while belop_start_ink_he < b_20_he:

        # (*) Oppdater el med den økende verdien slik at neste el i _matrix_jp_null_ls kan brukes (lager variasjonen)
        _mat_matrix[2][0] = belop_start_ink_he

        # Spinner
        if is_spinning == 1: i_spin = mt.spinner(i_spin)

        # Debug > Lang utregning
        if belop_debug == 1: print(f"{_B_start_str} :: ", belop_start_ink_he)

        # Resetting
        tab_mat = mt.deepcopy(tab_mat_0)

        # Tøm terminal-skjerm for hvert run
        mt.system("clear")

        # Lager tabell
        tab_mat = mt.tabell_økonomi_lag(
            tab_mat             = tab_mat,
            tab_desimal         = None,                 # None, -1
            tab_print           = 1,
            tab_bredde          = tab_bredde,
            tab_is_farge        = tab_is_farge,
            tab_farge_typ       = tab_farge_typ_mat,    # Default: Hvit
            tab_fx              = _mat_matrix,          # Default: Ingen effekt
            vf                  = vf_he,
            n_periode           = n_he,
            aar_start           = aar_start_he,
            innskudd            = innskudd_he,
            innskudd_desimal    = 2,                    # -1, None,
            innskudd_d          = 0,
            innskudd_d_desimal  = -1,                   # -1, None
            b_start_desimal     = -1,                   # -1, None
            b_slutt_desimal     = -1,                   # -1, None
        )

        # Finn elementet
        belop_start_ink_he = mt.tabell_finn_el(tab_mat, _n_periode, _B_start_str)

        # Øker det neste innskudd_d_mi-argumentet neste tabell-iterasjon
        innskudd_he += 1

    # Dekrement med 1 for å få forrige innskudd_d_mi
    innskudd_he -= 1

    # Print tabell og svar-setninger
    # mt.oppg_strek(1)
    print("")
    _oppg_txt_s = mt.tabell_print_farge(f"------------------------------------------", tab_farge_typ_svar)
    _oppg_txt_1 = mt.tabell_print_farge(f"Oppg c) Hermod - While-løkke", tab_farge_typ_svar)
    _oppg_txt_2 = mt.tabell_print_farge(f"- Hermod må ha et årlig innskudd på ca. {innskudd_he} kr", tab_farge_typ_svar)
    _oppg_txt_3 = mt.tabell_print_farge(f"  for å få {b_20_he} kr like etter innskudd {_n_periode}", tab_farge_typ_svar)
    print(_oppg_txt_s)
    print(f""); print(_oppg_txt_1); print("")
    mt.tabell_print( # mt
        tab_bredde          = tab_bredde,
        tab_mat             = tab_mat,
        tab_print_1_start   = tab_print_1_start,
        tab_print_1_slutt   = tab_print_1_slutt,
        tab_print_2_start   = tab_print_2_start,
        tab_print_2_slutt   = tab_print_2_slutt,
        tab_is_farge        = tab_is_farge,
        tab_farge_typ       = tab_farge_typ_svar,
        n_periode           = _n_periode
    )
    print(f"")
    print(_oppg_txt_2)
    print(_oppg_txt_3)

# b) Hermod > 692 852 kr like etter innskudd 20 > import matematikk (CAS)
if _blokk == -1:

    # Variabler
    _n_periode = mt.sjekk_datatype(vari_navn = "_n_periode", vari = n_he, typ = int)

    # Geometrisk formel: B_n_mi_2 = b_1 * k ** (n - 1)
    for n in range(1, _n_periode + 1):
        B_n_he     = (INSK_he) * vf_he ** (n - 1)
        B_sum_he   += B_n_he

    # Løser likningen med superløs()
    insk = mt.superløs(variabel = INSK_he,
                       vs       = B_sum_he,
                       hs       = b_20_he,
                       rund     = tab_desimal,
                       debug    = -1)

    # Print svar-setninger
    mt.oppg_strek(1)
    print(f""); print(f"Oppg b) Hermod - import matematikk"); print(f"")
    print(f"- Hermod må ha et årlig innskudd på ca. {insk} kr")
    print(f"  for å få ca. {b_20_he} kr like etter innskudd {_n_periode} ")

# b) Hermod > 692 852 kr like etter innskudd 20 > import sympy (CAS)
if _blokk == -1:

    # Import
    import sympy as sp

    # Resett
    _n_periode  = mt.sjekk_datatype(vari_navn = "_n_periode", vari = n_he, typ = int)
    INSK_he     = sp.Symbol("IS__he")   # Hermod: CAS-variabel for årlig innskudd
    b_20_he     = 692852                # Hermod: Beløps-grense like etter innskudd nr. 20
    B_n_he      = sp.Symbol("B_n_he")   # Hermod: CAS-variabel for beløp nr. n
    B_sum_he    = 0                     # Hermod: CAS-variabel for summen av beløper

    # Geometrisk formel: B_n_mi_2 = b_1 * k ** (n - 1)
    for n in range(1, _n_periode + 1):
        B_n_he     = (INSK_he) * vf_he ** (n - 1)
        B_sum_he   += B_n_he

    # Lager likning
    likning_he = sp.Eq(B_sum_he, b_20_he)

    # Løser likningen
    insk_ls_he = sp.solve(likning_he, INSK_he, domain = sp.Reals)

    # Henter verdien i fra løsnings-settet
    insk_el_he = round(insk_ls_he[0], tab_desimal) # Her ble løsningen en liste

    # Print svar-setninger
    mt.oppg_strek(1)
    print(f""); print(f"Oppg b) Hermod - import sympy"); print(f"")
    print(f"- Hermod må ha et årlig innskudd på ca. {insk_el_he} kr")
    print(f"  for å få ca. {b_20_he} kr like etter innskudd {_n_periode} ")

# c) Miriam > Millionær > Manuell brute force med binært søk
if _blokk == -1:

    # Resetting
    tab_mat = mt.deepcopy(tab_mat_0)

    # 1 000 000: Prøver verdier fra 0 til 1836.33
    innskudd_d_mi = 1836.33 # 1655 (rente = 4)

    # Lager tabell
    tab_mat = mt.tabell_økonomi_lag(
        tab_mat             = tab_mat,
        tab_desimal         = -1,                   # None, -1
        tab_print           = -1,
        tab_bredde          = tab_bredde,
        vf                  = vf_mi,
        n_periode           = n_mi,
        aar_start           = aar_start_mi,
        innskudd            = innskudd_mi,
        innskudd_desimal    = 2,                    # -1, None,
        innskudd_d          = innskudd_d_mi,
        innskudd_d_desimal  = -1,                   # -1, None
        b_start_desimal     = None,                 # -1, None
        b_slutt_desimal     = None,                 # -1, None
    )

    # Finn start-beløpet
    tab_mat_el = mt.tabell_finn_el(tab_mat, n_mi, _B_start_str)

    # Print tabell og svar-setninger
    mt.oppg_strek(1)
    print(f""); print(f"Oppg c) Miriam: Manuell brute force med binært søk"); print("")
    mt.tabell_print(
        tab_mat             = tab_mat,
        tab_print_1_start   = tab_print_1_start,
        tab_print_1_slutt   = tab_print_1_slutt,
        tab_print_2_start   = tab_print_2_start,
        tab_print_2_slutt   = tab_print_2_slutt,
        tab_bredde          = tab_bredde,
        n_periode           = n_mi
    )
    print(f"")
    print(f"- Miriam må øke innskuddet med nøyaktig {innskudd_d_mi} kr")
    print(f"  for å få {tab_mat_el} kr like etter innskudd {n_mi} ")

# c) Miriam > Millionær > While-løkke
if _blokk == -1:

    # While-løkken kjører så lenge beløpet er mindre enn 1 000 000
    _n_periode          = mt.sjekk_datatype(vari_navn = "n_periode", vari = n_mi, typ = int)
    innskudd_d_mi       = 0 # 1655 # 1837
    belop_start_ink_mi  = 0

    # Kjør løkken så lenge start-beløpet nr. 20 er mindre enn en mill
    while belop_start_ink_mi < b_20_mi_2:

        # Resetting
        tab_mat = mt.deepcopy(tab_mat_0)

       # Regner ut tabellen
        tab_mat = mt.tabell_økonomi_lag(
            tab_mat             = tab_mat,
            tab_desimal         = None,             # None, -1
            tab_print           = -1,
            tab_bredde          = tab_bredde,
            vf                  = vf_mi,
            n_periode           = _n_periode,
            aar_start           = aar_start_mi,
            innskudd            = innskudd_mi,
            innskudd_desimal    = -1,               # -1, None,
            innskudd_d          = innskudd_d_mi,
            innskudd_d_desimal  = -1,               # -1, None
            b_start_desimal     = -1,               # -1, None
            b_slutt_desimal     = -1,               # -1, None
        )

        # Finn elementet
        belop_start_ink_mi = mt.tabell_finn_el(tab_mat, _n_periode, _B_start_str)

        # Øker det neste innskudd_d_mi-argumentet neste tabell-iterasjon
        innskudd_d_mi += 1

    # Dekrement med 1 for å få forrige innskudd_d_mi
    innskudd_d_mi -= 1

    # Print tabell og svar-setninger
    mt.oppg_strek(1)
    print(f""); print(f"Oppg c) Miriam - While-løkke"); print(f"")
    mt.tabell_print(
        tab_bredde          = tab_bredde,
        tab_mat             = tab_mat,
        tab_print_1_start   = tab_print_1_start,
        tab_print_1_slutt   = tab_print_1_slutt,
        tab_print_2_start   = tab_print_2_start,
        tab_print_2_slutt   = tab_print_2_slutt,
        n_periode           = n_mi
    )
    print(f"")
    print(f"- Miriam må øke innskuddet med ca. {innskudd_d_mi} kr")
    print(f"  for å få ca. {b_20_mi_2} kr like etter innskudd {_n_periode} ")

# c) Miriam > Millionær > import matematikk (CAS)
if _blokk == -1:

    # Resetting
    _n_periode = mt.sjekk_datatype(vari_navn = "_n_periode", vari = n_mi, typ = int)

    # Geometrisk formel: B_n_mi_2 = b_1 * k ** (n - 1)
    for n in range(1, _n_periode + 1):
        B_n_mi_2     = (innskudd_0_mi + (n - 1) * INSK_d_mi) * vf_mi ** (n - 1)
        B_sum_mi_2   += B_n_mi_2

    # Løser likningen med superløs()
    inn_d_val = mt.superløs(variabel = INSK_d_mi,
                            vs       = B_sum_mi_2,
                            hs       = b_20_mi_2,
                            rund     = tab_desimal,
                            debug    = -1)

    # Print svar-setninger
    mt.oppg_strek(1)
    print(f""); print(f"Oppg c) Miriam - import matematikk"); print(f"")
    print(f"- Miriam må øke innskuddet med ca. {inn_d_val} kr")
    print(f"  for å få ca. {b_20_mi_2} kr like etter innskudd {n_mi} ")

# c) Miriam > Millionær > import sympy (CAS)
if _blokk == -1:

    # Import
    import sympy as sp

    # Resett
    INSK_d_mi           = mt.Symbol("INSK_d_mi")    # Miriam: CAS-variabel for fast øking av innskudd
    B_n_mi_2            = mt.Symbol("B_n_mi_2")     # Miriam: CAS-variabel for beløp nr. n
    B_sum_mi_2          = 0                         # Miriam: CAS-variabel for summen av beløper
    b_20_mi_2           = 1000000                   # Miriam: Beløps-grense like etter innskudd nr. 20
    _n_periode          = mt.sjekk_datatype(vari_navn = "_n_periode", vari = n_mi, typ = int)

    # Geometrisk formel: B_n_mi_2 = b_1 * k ** (n - 1)
    for n in range(1, _n_periode + 1):
        B_n_mi_2     = (innskudd_0_mi + (n - 1) * INSK_d_mi) * vf_mi ** (n - 1)
        B_sum_mi_2   += B_n_mi_2

    # Lager likning
    likning_mi = sp.Eq(B_sum_mi_2, b_20_mi_2)

    # Løser likningen
    insk_ls_mi = sp.solve(likning_mi, INSK_d_mi, domain = sp.Reals)

    # Henter verdien i fra løsnings-settet
    insk_el_mi = round(insk_ls_mi[0], tab_desimal) # Her ble løsningen en liste

    # Print svar-setninger
    mt.oppg_strek(1)
    print(f""); print(f"Oppg c) Miriam - import sympy"); print(f"")
    print(f"- Miriam må øke innskuddet med ca. {insk_el_mi} kr")
    print(f"  for å få ca. {b_20_mi_2} kr like etter innskudd {n_mi} ")




# 🚀 programmering.no | 🤓 matematikk.as
# - Prøv å add et element til frozenset (går ikke)

# Variabler
sett = frozenset({"bil", "tog", "sykkel"})

# Print originalt sett
print(f"Originalt sett: {sett}")

# Prøv å add et element til frozenset (går ikke)
sett.add("fly")

# Print oppdatert sett
print(f"Oppdatert sett: {sett}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Rotere en liste til høyre eller venstre
# - Def: Listene roterer som "Pacman"
#   Å f.eks. rotere med 3 til venstre betyr at 3 elementer først "forsvinner"
#   til venstre, før de dukker opp igjen på høyre side
# - Eks: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10] => [4, 5, 6, 7, 8, 9, 10, 1, 2, 3]

# Input
liste 	         = ["Superman", "Batman", "Wolverine", "Spiderman", "Iron Man"]
rotasjon_retning = "h"
rotasjon_antall  = 2

# Funksjon som roterer listen til ventre eller høyre
def roter_liste(liste, rotasjon_retning, rotasjon_antall):

	rota_ant       = int(rotasjon_antall)
	rotert_liste_1 = list()
	rotert_liste_2 = list()
	ferdig_liste   = list()

	i = 0
	while i < len(liste):

        # Eks. 3 venstre-rotert => [4, 5, 6, 7, 8, 9, 10] + [1, 2, 3]
		if rotasjon_retning == "v":
			if i <  rota_ant: 			   rotert_liste_1.append(liste[i]) # Eks: [1, 2, 3]
			if i >= rota_ant: 			   rotert_liste_2.append(liste[i]) # Eks: [4, 5, 6, 7, 8, 9, 10]

        # Eks. 3 høyre-rotert => [8, 9, 10] + [1, 2, 3, 4, 5, 6, 7]
		if rotasjon_retning == "h":
			if i >= len(liste) - rota_ant: rotert_liste_2.append(liste[i]) # Eks: [8, 9, 10]
			if i <  len(liste) - rota_ant: rotert_liste_1.append(liste[i]) # Eks: [1, 2, 3, 4, 5, 6, 7]

        # Inkrement i
		i += 1

	# Legg sammen listene
	ferdig_liste =  rotert_liste_2 + rotert_liste_1

	return ferdig_liste

# Roterte lister
rotert_v_liste = roter_liste(liste, "v", rotasjon_antall) # Eks: ([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], "v", 3)
rotert_h_liste = roter_liste(liste, "h", rotasjon_antall)

# Print
print(f"Liste før rotasjon   	       : {liste}")
if rotasjon_retning == "v":
	print(f"Liste etter venstre-rotasjon ({rotasjon_antall}) : {rotert_v_liste}")
if rotasjon_retning == "h":
	print(f"Liste etter right-rotasjon ({rotasjon_antall}) : {rotert_h_liste}")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 c) og d) Etterspørsel 
# - Løser oppgaven med CAS i Python
# - Kopier denne kommandoen i terminalen for å importere matematikk: 
#   $ pip install matematikk

import matematikk as mt

# Variabler
_blokk = 1
if _blokk == 1:

    # Variabler
    x                   = mt.Symbol("x")        # Etterspørsel (antall produserte enheter)
    p                   = mt.Symbol("p")        # Pris per enhet
    I                   = mt.Symbol("I")        # Inntekt
    K                   = mt.Symbol("K")        # Kostnad
    svar_c_ls           = list()                # Svar-setninger
    svar_d_ls           = list()                # Svar-setninger
    x_desimal           = None                  # Avrunding (heltall)
    desimal_x_opt       = None                  # Avrunding
    p_desimal           = 1                     # Avrunding

# Felles: Oppg c) og d)
if _blokk == 1:

    # Definerer uttrykket for pris-funksjonen, p(x)
    p = 79 - 12.2 * mt.ln(x)

    # Definerer uttrykket for inntekts-funksjonen, I(x)
    I = x * p

    # Deriverer I mhp. x og får dI = 66.8 - 12.2*log(x)
    dI = mt.deriver(I, x) # I'(x)

# Oppg c)
if _blokk == 1:

    # Løser likningen dI = 0
    x_inntekt_max = mt.superløs(variabel = x,
                                vs       = dI,
                                hs       = 0,
                                rund     = x_desimal,
                                debug    = -1)

    # Setter x_inntekt_max = 239 inn i p og definerer det nye uttrykket som p_inntekt_max
    p_inntekt_max = p.subs(x, x_inntekt_max)

    # Henter verdien til p_inntekt_max
    p_inntekt_max_val = p_inntekt_max.evalf()

    # Runder av 12.1871446664356 -> 12.20
    p_inntekt_max_val = round(p_inntekt_max_val, p_desimal)

    # Svar-setninger
    svar_c_ls.append(f"")
    svar_c_ls.append(f"Oppg c)")
    svar_c_ls.append(f"")
    svar_c_ls.append(f"- Inntekten er størst når etterspørselen er ca. {x_inntekt_max} enheter")
    svar_c_ls.append(f"- Dette gir en pris på ca. {p_inntekt_max_val} kr")

    # Print svar-setninger
    for svar in svar_c_ls: print(svar)

# Oppg d)
if _blokk == 1:

    # Definerer uttrykket for kostnads-funksjonen, K(x)
    K = 0.021 * x**2 + 10 * x + 910

    # Deriverer K mhp. x og får dK = 0.042*x + 10
    dK = mt.deriver(K, x) # K'(x)

    # Løser likningen dI = dK mhp. x
    x_opt = mt.superløs(variabel = x,
                        vs       = dI,
                        hs       = dK,
                        rund     = desimal_x_opt,
                        debug    = -1)

    # Svar-setninger
    svar_d_ls.append(f"")
    svar_d_ls.append(f"Oppg d)")
    svar_d_ls.append(f"")
    svar_d_ls.append(f"- Grense-inntekten er lik grense-kostnaden ved {x_opt} enheter")
    svar_d_ls.append(f"- Dette betyr at {x_opt} enheter gir størst overskudd fordi:")
    svar_d_ls.append(f"     O`(x) = 0, max overskudd")
    svar_d_ls.append(f"     O`(x) = I`(x) - K`(x), definisjonen av overskudd")
    svar_d_ls.append(f"     0 = I`(x) - K`(x)")
    svar_d_ls.append(f"     K`(x) = I`(x)")

    # Print svar-setninger
    for svar in svar_d_ls: print(svar)



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Torodd)
# Oppgave 1 b) Sofa-produksjon til møbelfabrikk - Størst overskudd 

from sympy import Symbol, diff, Eq, Reals, solveset

# definerer x
x = Symbol("x")

O = -0.041*x**2 +11*x -103

# definerer vs og hs av likninga
vs = diff(O)    #O'(x)
hs = 0

# set opp likninga, og løyser ho
likning = Eq(vs, hs)
løysing = solveset(likning, x, domain=Reals)

# skriv ut svaret
print(f"x = {løysing.evalf(4)}")



def vekstfaktor(fortegn = str(),
                p       = float(),
                desimal = -1,
                rund    = -1):

    # Gjør argument-alias om til samme variabel
    _desimal = -1
    if rund     != -1: _desimal = rund
    if desimal  != -1: _desimal = desimal
 
    # Vekstfaktor er definert som V = 1 ± p / 100, p: prosentvis vekst [%]
    v = float()
    if fortegn == "+": v = 1 + p / 100 # "+": Øker
    if fortegn == "-": v = 1 - p / 100 # "-": Minker

    # Runder av svaret
    if _desimal != -1: v = round(v, _desimal)

    return v




# yooo
def sjekk_datatype(vari_navn = str(), vari = None, typ = None):

    # None er default-argument pga. forskjellige datatyper

    # Riktig type
    if type(vari) == typ:

        return vari

    # Feil type
    if type(vari) != typ:

        # Advarsel
        _typ = type(vari)

        # int: Type caster feil type -> riktig type
        if typ == int: _vari = int(vari)

        print(f"Advarsel ({vari_navn}): Type caster {vari} ({_typ.__name__}) til {_vari} ({typ.__name__})")

        return _vari

# Alias > 1 > ...

# Alias > 2 > ... 
datatype_sjekk                        = sjekk_datatype



# Versjon: 0.0.209

# '''

# os
from os import(
    system,
    )

# copy
from copy import(
    deepcopy,
    )

# numpy
from numpy import(
    random,
    mean,
    sqrt,

    polyfit,
    )

# sympy
from sympy import (
    ConditionSet,
    core,
    diff,
    Eq,
    FiniteSet,
    Intersection,
    ln,
    log,
    nsolve,
    Reals,
    solve,
    solveset,
    Symbol,
    )

# colorama
from colorama import(
    Style,
    Fore,
    )

# DEV
# - Dobbelsjekk alle export-funk med her og faktisk fil-liste
# - Sjekk import innsiden / utsiden av funk


##########################################
# import matematikk
##########################################

##########################################
# DEV > ... > Ingen dependencies
##########################################

# sjekk_datatype()
from ._exports.def_sjekk_datatype_mas import (
    sjekk_datatype,

    # Alias > 1 > ...

    # Alias > 2 > ...
    datatype_sjekk,
    )

# oppg_strek()
from ._exports.def_oppg_strek_mas import (
    oppg_strek,

    # Alias > 1 > ...

    # Alias > 2 > ...
    strek_oppg,
    )

# spinner()
from ._exports.def_spinner_mas import (
    spinner,
    )

##########################################
# DEV > ... > Dependencies > Skriv på!
##########################################

# sannsynlighet_x_fra_mu_sd()
# - Dep > matematikk
from ._exports.def_normalfordeling_standard_mas import (
    sannsynlighet_x_fra_mu_sd,
    sannsynlighet_x_mindre_enn_fra_mu_sd,
    sannsynlighet_gjennomsnitt_x_fra_mu_sd,

    # Alias > 1 > sannsynlighet_x_fra_mu_sd
    sannsynlighet_X_fra_mu_sd,
    sannsynlighet_k_fra_mu_sd,
    sannsynlighet_gunstig_fra_mu_sd,

    # Alias > 1 > sannsynlighet_x_mindre_enn_fra_mu_sd
    sannsynlighet_x_storre_enn_fra_mu_sd,

    # Alias > 1 > sannsynlighet_gjennomsnitt_x_fra_mu_sd
    sannsynlighet_gjennomsnitt_x_mindre_fra_mu_sd,
    )

# tabell / funks
from ._exports.def_tabell_mas import (
    tabell_print_effekt,
    tabell_print_farge,
    tabell_print_kolonne_padding,
    tabell_print_rad,
    tabell_print,
    tabell_finn_el,
    tabell_økonomi_lag,

    # Alias > 1
    tabell_okonomi_lag,
    )

# deriver()
from ._exports.def_deriver_mas import (
    deriver,

    # Alias > 1
    derivert,
    momentan_vekst,
    momentan_vekstfart,
    )

# DEV > Fix filnavn og linjen under til _mas slik at ulik fra funk
# vekstfaktor()
from ._exports.def_vekstfaktor import (
    vekstfaktor
    )

# DEV > Fix filnavn og linjen under til _mas slik at ulik fra funk
# vekstfaktor_cas()
from ._exports.def_vekstfaktor_cas import (
    vekstfaktor_cas,
    )

# reggis()
from ._exports.def_reggis_matematikk_mas import (
    reggis,

    # Alias > 1
    reggis_cas,
    regresjon,
    regresjon_cas,
    regresjon_polynom,
    regresjon_polynom_cas,

    # Alias > 2
    cas_regresjon,
    cas_regresjon_polynom,
    regresjon_polynom_cas,
    )

# superlos()
from ._exports.def_superlos_matematikk_mas import (
    superløs,

    # Alias > 1
    los,
    losning,
    løs,
    løsning,
    superlos,
    super_los,
    super_løs,

    # Alias > 2
    los_super,
    løs_super,
    )

# ekstremalpunkt_max()
from ._exports.def_ekstremalpunkt_max_mas import (
    ekstremalpunkt_max,

    # Alias > 1
    ekstremalpunkt_maks,
    ekstremalpunkt_maksimalt,
    toppunkt,

    # Alias > 2 > ...
    )

# overskudd_max()
from ._exports.def_enhet_fra_overskudd_max_mas import (
    enhet_fra_overskudd_max,

    # Alias > 1
    enhet_fra_overskudd_maks,
    enhet_fra_overskudd_maksimalt,
    enhet_fra_overskudd_mest,
    enhet_fra_overskudd_storst,
    enhet_fra_overskudd_størst,

    # Alias > 2
    enhet_fra_maks_overskudd,
    enhet_fra_maksimalt_overskudd,
    enhet_fra_mest_overskudd,
    enhet_fra_storst_overskudd,
    enhet_fra_størst_overskudd,
    )

# enhet_fra_overskudd_max_med_kostnad()
from ._exports.def_enhet_fra_overskudd_max_med_kostnad_mas import (
    enhet_fra_overskudd_max_med_kostnad,

    # Alias > 1 ...
    enhet_fra_overskudd_max_kostnad,

    # Alias > 2 > ...
    enhet_fra_max_overskudd_med_kostnad,
    enhet_fra_max_overskudd_kostnad,
    )

# enhet_og_pris_fra_inntekt_max()
from ._exports.def_enhet_og_pris_fra_inntekt_max_mas import (
    enhet_og_pris_fra_inntekt_max,

    # Alias > 1
    enhet_og_pris_fra_inntekt_max,

    # Alias > 2 > ...
    )

# Alle svar
# økonomi()
from ._exports.def_alle_svar_okonomi_mas import (
    økonomi,

    # Alias > 1
    # enhet_og_pris_fra_inntekt_max,

    # Alias > 2 > ...
    )

# Alle svar
# eksamen()
from ._exports.def_alle_svar_eksamen_mas import (
    eksamen,

    # Alias > 1
    # enhet_og_pris_fra_inntekt_max,

    # Alias > 2 > ...
    )

# '''



# 🚀 programmering.no | 🤓 matematikk.as
# - Transponer matrise med numpy

import numpy

# Matrise (datatype-format)
matrise = [[1, 2, 3], [4, 5, 6]]

# Print
print("")
print("Matrise (datatype-format):")
print(matrise)

# Matrise (matematisk format)
matrise = numpy.array(matrise)

# Print
print("")
print("Matrise (matematisk format):")
print(matrise)

# Print
print("")
print("Transponert matrise (matematisk format):")
print(matrise.T)



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 c) Sofa-produksjon til møbelfabrikk - Minst salgspris 

from numpy import polyfit
from sympy import Eq, diff, Reals, solveset, Symbol

# Variabler
x              = Symbol("x")           # Produksjons-mengden x
x_max          = float()               # Produksjonsmengden som gir størst overskudd
x_liste        = [                     # Liste med x-verdier
    10,
    25,
    40,
    70,
    100,
    140,
    180]
y_liste        = [                     # Liste med y-verdier
    270,
    550,
    870,
    1500,
    2200,
    3300,
    4500]
K              = Symbol("K")           # Kostnads-funksjonen
grad           = 2                     # Polynomets grad
O              = Symbol("O")           # Overskudds-funksjonen
dO             = Symbol("dO")          # Den deriverte av overskudds-funksjonen
pris_ny        = Symbol("pris_ny")     # Ny salgspris
I              = Symbol("I")           # Inntekts-funksjonen
rund           = 2                     # -1: Ingen avrunding, 0: Eks: 123.0, n > 0: n siffer etter desimaltegnet, None: Heltall
_blokk         = 1                     # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker

# Definerer funksjoner
def reggis(variabel = Symbol(""),
           grad     = int(),
           x_liste  = list(),
           y_liste  = list(),
           rund     = -1):

    # - Lager polynom-funksjon av valgfri grad vha. regresjon (CAS)

    # Variabler
    _koeff_liste       = list()                # Liste med koeffesienter i fra polyfit()
    _koeff             = float()               # Polynomets koeffisienter
    _ledd              = Symbol("_ledd")       # Polynomets ledd
    _ledd_grad         = int()                 # Polynomets ledd inkludert grad
    _uttrykk           = Symbol("_uttrykk")    # Polynomets uttrykk (hs)

    # polyfit() returnerer en liste med regresjons-koeffesientene
    _koeff_liste = polyfit(x_liste, y_liste, grad)

    # Hver iterasjon lager det neste leddet i polynomet
    for i in range(len(_koeff_liste)):

        # - På 1. iterasjon må _uttrykk nulles ut slik at det ikke blir en
        #   egen variabel i uttrykket, f.eks. "_uttrykk + 0.04*x**2 + 17.05*x + 102.7"
        if i == 0: _uttrykk = 0

        # Runder av koeffesienten
        if rund == -1: _koeff = _koeff_liste[i]
        if rund != -1: _koeff = round(_koeff_liste[i], rund)

        # Lager leddene i fra størst til minst grad
        _ledd_grad = grad - i

        # Formaterer leddene
        if _ledd_grad >= 2: _ledd = _koeff * variabel**_ledd_grad
        if _ledd_grad == 1: _ledd = _koeff * variabel
        if _ledd_grad == 0: _ledd = _koeff

        # Legger det nye leddet til polynomet
        _uttrykk += _ledd

    return _uttrykk

# a) Finn O(x)
if _blokk == 1:

    # Lager polynom-funksjon av valgfri grad vha. regresjon (CAS)
    K = reggis(variabel = x,
                  grad     = grad,
                  x_liste  = x_liste,
                  y_liste  = y_liste,
                  rund     = rund)

    # Svar-setninger
    svar_a_liste = list()
    svar_a_liste.append(f"")
    svar_a_liste.append(f"Oppg a)")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"- Med regresjon fra numpy (polyfit) ser vi at hvis bedriften")
    svar_a_liste.append(f"  produserer x enheter, så vil")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"    {K}")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"  være en god model for det måntlige overskuffet (i tusen kroner)")

    # Print svar-setninger
    for svar in svar_a_liste: print(svar)

# b) Størst overskudd
if _blokk == 1:

    # Definerer uttrykket for overskudds-funksjonen, O
    O = -0.041*x**2 + 11*x - 103

    # Deriverer O mhp. x og får dO = 11 - 0.082*x
    dO = diff(O, x) # O'(x)

    # Lager likningen dO = 0 med Eq(vs, hs) 
    likning = Eq(dO, 0)

    # Løser likningen for x med solveset() og får løsnings-settet {134.146341463415}
    x_topp_set = solveset(likning, x, domain=Reals) # Løsnings-domene i ℝ (Reals)

    # Henter elementet i fra løsnings-settet, 134.146341463415
    x_max = x_topp_set.args[0]

    # Runder av 134.146341463415 -> 134
    x_max = round(x_max, None)

    # Svar-setninger
    svar_b_liste = list()
    svar_b_liste.append(f"")
    svar_b_liste.append(f"Oppg b)")
    svar_b_liste.append(f"")
    svar_b_liste.append(f"- Negativ a-koeffisient gir et toppunkt på O")
    svar_b_liste.append(f"- Produksjonsmengden x = {x_max} gir størst overskudd")

    # Print svar-setninger
    for svar in svar_b_liste: print(svar)

# c) Minst salgspris
if _blokk == 1:

    # Definerer uttrykkene for kostnads- og inntekts-funksjonen, K og I
    K = 0.041*x**2 + 17*x + 103
    I = pris_ny*x

    # Definerer uttrykket for overskudds-funksjonen, O = pris_ny*x - 0.041*x**2 - 17*x - 103
    O = I - K # Neg. a-koeff (-0.041) -> 'sur graf' -> Toppunkt

    # Deriverer O mhp. x og får dO = pris_ny - 0.082*x - 17
    dO = diff(O, x)

    # Lager likningen dO = 0 med Eq(vs, hs) 
    likning = Eq(dO, 0)

    # Løser likningen for x med solveset() og får løsnings-settet Intersection({12.19*pris_ny - 207.31}, Reals)
    x_topp_set = solveset(likning, x, domain=Reals)

    # Henter elementet i fra løsnings-settet, 12.195*pris_ny - 207.317
    x_max = x_topp_set.args[1].args[0] # x_topp = 12.2*pris_ny - 207 gir topp-punktet på O 

    # Setter x_max = 12.195*pris_ny - 207.317 inn i O og definerer det nye uttrykket som O_topp 
    O_topp = O.subs(x, x_max)

    # Lager likningen O_topp = 1000 med Eq(vs, hs) 
    likning = Eq(O_topp, 1000) # kr 1 000 000 "i antall tusen kr" er 1000

    # Løser likningen for pris_ny med solveset() og får løsnings-settet (3.55039034023664, 30.4496096597634)
    pris_ny_set = solveset(likning, pris_ny, domain=Reals)

    # Henter elementene i fra løsnings-settet, 3.5503, 30.4496
    pris_ny_1 = round(pris_ny_set.args[0], 3) # Runder av 3.5503 -> 3.55
    pris_ny_2 = round(pris_ny_set.args[1], 3) # Runder av 30.4496 -> 30.45

    # Setter pris_ny = 3.55 inn i x_max og definerer det nye uttrykket som x_topp_1 
    x_topp_1 = x_max.subs(pris_ny, pris_ny_1) # x_topp_1 = -164.02
    x_topp_1 = round(x_topp_1, None) # Runder av -164.02 -> -164

    # Setter pris_ny = 30.45 inn i x_max og definerer det nye uttrykket som x_topp_2 
    x_topp_2 = x_max.subs(pris_ny, pris_ny_2) # x_topp_2 = 164.02
    x_topp_2 = round(x_topp_2, None) # Runder av 164.02 -> 164

    # Ganger med 1000 for å få salgsprisen i kr (og ikke i "antall 1000 kr")
    pris_ny_1_kr = pris_ny_1 * 1000
    pris_ny_2_kr = pris_ny_2 * 1000

    # Svar-setninger
    svar_c_liste = list()
    svar_c_liste.append(f"")
    svar_c_liste.append(f"Oppg c)")
    svar_c_liste.append(f"")
    svar_c_liste.append(f"- En salgspris på kr {pris_ny_1_kr} gir x = {x_topp_1} produserte sofaer")
    svar_c_liste.append(f"- En salgspris på kr {pris_ny_2_kr} gir x = {x_topp_2} produserte sofaer")
    svar_c_liste.append(f"")
    svar_c_liste.append(f"- Vi kan ikke kan ha et negativt antall produserte sofaer")
    svar_c_liste.append(f"")
    svar_c_liste.append(f"- I denne oppgaven blir derfor riktig salgspris kr {pris_ny_2_kr} per sofa")

    # Print svar-setninger
    for svar in svar_c_liste: print(svar)



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (Matematikk AS)
# Oppgave 3 a) Coca-Cola vs. Pepsi-Cola - 10 glass 

from scipy.stats import binom

n                   = 10      # Forsøk
p                   = 0.5     # Sannsynlighet
x_liste             = []      # Liste med verdiene til x
fordeling_liste     = []      # Liste med sannsynlighetsfordelingen til x
riktig              = 6       # Antall riktige
desimal             = 3       # Antall desimaler i avrunding

# Legger alle mulige x-verdier i listen: 0 -> 10
for i in range(n + 1):
    x_liste.append(i)

# Lager en liste med sannsynlighetsfordelingen
fordeling_liste = binom.pmf(x_liste, n, p)

# Lager en liste med sannsynlighetsfordelingen
fordeling_liste[riktig] = round(fordeling_liste[riktig], desimal)

# Print svaret
print(f"Sannsynlighet for å gjette 6 riktige: {fordeling_liste[riktig]}")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (Matematikk AS)
# Oppgave 3 b) Coca-Cola vs. Pepsi-Cola - Hypotese-test 

from scipy.stats import binom
import numpy as np

_blokk = 1
if _blokk == 1:
    n                       = 10      # Antall forsøk
    p                       = 0.5     # Sannsynlighet (sannsyn) for riktig cola-type
    x_liste                 = []      # Liste med alle mulige x-verdier
    x_riktig                = 8       # Antall riktige
    fordeling_liste         = []      # Liste med sannsynlighetsfordelingen til x
    fordeling_liste_kum     = []      # Liste med kumulativ sannsynlighetsfordeling
    sannsyn_riktig_kum      = 0       # Sannsyn for å gjette X > riktige
    konfidens               = 0.05    # Konfidens
    desimal                 = 3       # Antall desimaler i avrunding
    _debug_fordeling        = 1       # 0: Ikke debug, 1: Debug
    _debug_fordeling_kum    = 0       # 0: Ikke debug, 1: Debug

def _debugging(ls, txt):
    print(""); print(txt)
    for i in range(len(ls)): print(ls[i] * 100)

# Legger alle x-verdiene i listen: 0, 1, 2, ... 10 (0 er ingen rette)
for i in range(n + 1): x_liste.append(i)

# Liste med fordelingen: 0, 0, 4, 11, 20, 24, 20, 11, 4, 0, 0
fordeling_liste = binom.pmf(x_liste, n, p)

# Debugger fordelingen (%): Å gjette 1 riktig betyr også å gjette 9 feil (lav sannsynlighet)
if _debug_fordeling == 1: _debugging(fordeling_liste, "Fordeling")

# Liste med kumulativ fordeling: Større eller mindre => Summere flere sannsyn
fordeling_liste_kum = np.cumsum(fordeling_liste)

# Debugger kumulativ fordeling (%): Vokser til 100 % ("mindre eller lik alle")
if _debug_fordeling_kum == 1: _debugging(fordeling_liste_kum, "Kumulativ fordeling")

# Sannsyn for å gjette minst riktig 8 av 10 er 94 %
# Vi antar mao. her at hun ikke gjetter, men kan smake forskjell (på alle!)
# Derfor summerer vi sannsyn fra 8 til 10
# Fra 8 (og f.eks. ikke 9) fordi dette var "feilmarginen" i hennes smaks-kompetanse
sannsyn_riktig_kum = 1 - fordeling_liste_kum[x_riktig - 1] # - 1 fordi starter på 0

# Ser om kumulativ sannsynlighet for å gjette X > x_riktig er:
if sannsyn_riktig_kum  > konfidens:
    # 1. Større enn konfidens: Stor sannsynlighet for å gjette x_riktig => smaker ikke forskjell
    print("Det er ikke sannsynlig at Marte smaker forskjell.")
if sannsyn_riktig_kum <= konfidens:
    # 1. Mindre enn konfidens: Liten sannsynlighet for å gjette x_riktig => smaker forskjell
    print("Det er sannsynlig at Marte smaker forskjell.")



import os

def spinner(i_spin):

    # Tøm terminal-skjerm
    os.system("clear")

    i_spin += 1
    if i_spin == 0: print("|")
    if i_spin == 1: print("/")
    if i_spin == 2: print("-")
    if i_spin == 3: print("\\")
    if i_spin == 4: print("|")
    if i_spin == 5: print("/")
    if i_spin == 6: print("-")
    if i_spin == 7: print("\\")

    if i_spin == 7: i_spin = -1

    return i_spin



# 🚀 programmering.no | 🤓 matematikk.as
# - Ser om et element eksisterer i en liste

# Verdier
liste = ["elefant", "ulv", "rev"]
dyr   = "elefant"

# Ser om el eksisterer i listen
print(f"Liste: {liste}")
if dyr in liste:
    print(f"Ja, {dyr} eksisterer i listen")
else:
	print("Nei, elementet eksisterer ikke i listen")



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 5 c) Volum av prisme - Minste samlede areal til platene når kassen er 80 cm^3 

from sympy import Eq, Reals, core, Symbol, ConditionSet, FiniteSet, Intersection, solve, solveset, nsolve

# Konstanter
_blokk = 1 # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker

# Definerer funksjoner
def superløs(variabel = Symbol(""),
             vs       = Symbol(""),
             hs       = Symbol(""),
             likning  = list(),
             rund     = -1,
             debug    = -1):

    vis_datatype = True
    losning_set, losning_set_sub = set(), set()
    losning_liste, losning_element, losning_rund_liste, losning_ut = list(), list(), list(), list()
    losning_rund_status, sett_typ, likn_ant, los_ant, _v_avrund_typ = str(), str(), str(), str(), str()
    losning_rund = float()

    if type(likning) != list: likning_tmp = likning; likning = []; likning.append(likning_tmp)

    if len(likning) == 0:
        likn_ant = "En likning"
        if type(variabel) == type(list()): variabel = variabel[0]
        if type(vs) != core.mul.Mul and type(vs) == type(list()): vs = vs[0]
        if type(hs) != core.mul.Mul and type(hs) == type(list()): hs = hs[0]
        _likning = Eq(vs, hs); losning_set = solveset(_likning, variabel, domain = Reals)

        if len(likning) == 0 and type(losning_set) == FiniteSet:
            sett_typ = "FiniteSet";
            if len(losning_set) == 1: los_ant = "en løsning"
            if len(losning_set) > 1: los_ant = str(len(losning_set)) + " " + "løsninger"

        if len(likning) == 0 and type(losning_set) == ConditionSet:
            sett_typ = "ConditionSet"; losning_element = nsolve(_likning, variabel, 1)
            los_ant = "en numerisk løsning"; losning_set = []; losning_set.append(losning_element)

        if len(likning) == 0 and type(losning_set) == Intersection:
            sett_typ = "Intersection"; losning_set_sub = losning_set.args[1]
            losning_element = losning_set_sub.args; losning_set = [];
            for el in losning_element: losning_set.append(el); rund = -1
            if len(losning_set) == 1: los_ant = "en løsning som er et uttrykk"
            if len(losning_set) > 1: los_ant = str(len(losning_set)) + " " + "løsninger som er uttrykk"

    if len(likning) > 0:
        sett_typ = ""; likn_ant = "Flere likninger (likningssett)"
        losning_set = solve(likning, variabel, dict=True)
        for i in range(len(losning_set)):
            koordinat_verdi_liste = list(); koordinat_variabel_rund_liste = list()
            for koordinat_variabel, koordinat_verdi in losning_set[i].items():
                koordinat_verdi_liste.append(koordinat_verdi)
                losning_rund = round(float(koordinat_verdi), rund)
                koordinat_variabel_rund_liste.append(losning_rund)
            losning_liste.append(koordinat_verdi_liste); losning_rund_liste.append(koordinat_variabel_rund_liste)

    for losning in losning_set:
        if rund == -1:
            _v_avrund_typ = "(eksakt)"; losning_rund_status = "Nei"; losning_rund_liste = ""
            if len(likning) == 0: losning_liste.append(losning); losning_ut = losning_liste
            if len(likning) > 0: losning_ut = losning_liste

        if rund != -1:
            _v_avrund_typ = "(avrundet)"; losning_rund_status = "Ja"
            if len(likning) == 0:
                losning_liste.append(losning); losning_rund = round(losning, rund)
                losning_rund_liste.append(losning_rund); losning_ut = losning_rund_liste
            if len(likning) > 0: losning_ut = losning_rund_liste

    if len(losning_ut) == 1: losning_ut = losning_ut[0]

    if debug == 1:
        print("")
        print(f"*** Debug ****")
        if vis_datatype == True:
            print(f"Data-type                        :: {sett_typ}")
        print(f"Løsnings-type                    :: {likn_ant} med {los_ant} {_v_avrund_typ}")
        print(f"Løsnings-sett                    :: {losning_set}")
        print(f"Løsnings-element(er)             :: {losning_liste}")
        print(f"Avrunding (ja/nei)               :: {losning_rund_status}")
        print(f"Løsnings-elemente(er) avrundet   :: {losning_rund_liste}")
        print(f"-----------------------------------")
        print(f"Løsning ut (returnert) -----------> {losning_ut}")
        print(f"-----------------------------------")

    return losning_ut

# a) Største volum dersom sidene i bunnen er 5 dm (løsning med likninger)
if _blokk == 1:

    # Regner ut volum av prisme med likning når areal_maks, lengde og dybde er gitt

    # Skisse av kassen
    if _blokk == 0:
        #                    |------------
        #                    | .	       .
        #                    |  -	   V    -
        #                    |   .------------
        #                    _   |			 |
        #                     .  |	   A     |   h
        #                 s     _|			 |
        #                        .------------
        # 
        #                                s
        pass

    # Konstanter og CAS-variabler (symbol)
    V                  = Symbol("V")   # Prismets volum
    V_80               = 80   # Prismets volum i oppg c)
    A                  = Symbol("A")   # Arealet av platene
    A_max              = 120           # Maksimalt arealet av platene
    s                  = Symbol("s")   # Sidene i den kvadratiske grunnflaten
    s_5                = 5             # Lengden på sidene
    h                  = Symbol("h")   # Prismets høyde

    # Definerer uttrykkene for A og V
    A = s**2 + 4*s*h
    V = s**2 * h

    # Setter s = 5 inn i A og definerer det nye uttrykket som A_5 (slik at ikke A skrives over) 
    A_5 = A.subs(s, s_5)

    # Løser likningen 120 = 5**2 + 4*5*h 
    h_losning = superløs(variabel = h,
                         vs       = A_max,
                         hs       = A_5,
                         rund     = 2,
                         debug    = -1)

    # Setter h = -(s + 5)/4 inn i V og definerer det nye uttrykket som V_uttrykk 
    V_max = V.subs(s, s_5).subs(h, h_losning) # Bruker .subs flere ganger etter hverandre ("chain rule")

    # Print svar-setninger
    svar_a_0 = f"Oppg a)"
    svar_a_1 = f"- Det største volumet kassen kan ha er {V_max} dm^3"
    print("")
    print(svar_a_0)
    print(svar_a_1)

# b) Maksimalt volum av kassen
if _blokk == 1:

    # Maksimalt volum steg-for-steg
    if _blokk == 0:
        # 1. For å finne maksimalt volum til kassen, så må vi
        #    derivere V
        # 2. Vi finner som vanlig ekstremalpunktet til V ved å sette den
        #    deriverte lik 0 og løse denne likningen
        # 3. Men, fordi V foreløpig er en funksjon av både s og h, så må
        #    vi først gjøre om V til å kun være uttrykt med en variabel
        # 4. Vi kan mao. bruke konstanten for det maksimale arealet 120 i
        #    likningen for A til å skrive denne likningen om til å være
        #    h uttrykt med s
        # 5. Når vi så bytter ut ut h i likningen til V med dette uttrykket,
        #    så får vi V kun uttrykt med s
        pass

    # Løser likningen 120 = s**2 + s*5*h for h
    h_uttrykk = superløs(variabel = h,
                         vs       = A_max,
                         hs       = A,
                         rund     = 2,
                         debug    = -1) # h = -(s + 5)/4

    # Setter h = -(s + 5)/4 inn i V og definerer det nye uttrykket som V_uttrykk 
    V_uttrykk = V.subs(h, h_uttrykk) # Nå er V kun uttrykt med s, og kan derfor enkelt deriveres

    # Deriverer V mhp. s og får dV = 30 - 3*s**2/4
    dV = V_uttrykk.diff()

    # Løser likningen dV = 0 
    s_topp_eksakt = superløs(variabel = s,
                             vs       = dV,
                             hs       = 0,
                             rund     = -1,  # rund  = -1 gir eksakt verdi
                             debug    = -1)  # debug = 1  viser både eksakt og avrundet verdi med superløs()

    # Setter inn den positive av de to løsningene for s, -2*sqrt(10) og 2*sqrt(10), inn i V_uttrykk
    V_uttrykk_eksakt = V_uttrykk.subs(s, s_topp_eksakt[1]) # V_uttrykk = 40*sqrt(10)

    # Runder av svaret
    V_uttrykk_rund = round(V_uttrykk_eksakt, 1)

    # Print svar-setninger
    svar_b_0 = f"Oppg b)"
    svar_b_1 = f"- Eksakt lengde er s_1 = {s_topp_eksakt[0]} og s_2 = {s_topp_eksakt[1]} dm"
    svar_b_2 = f"- Fordi s er en lengde, så bruker vi kun den positive løsningen"
    svar_b_3 = f"- Dette gir et eksakt maksimalt volum på V = {V_uttrykk}"
    svar_b_4 = f"- Avrundet svar gir derfor løsningen V = {V_uttrykk_rund} dm^3"
    print("")
    print(svar_b_0)
    print(svar_b_1)
    print(svar_b_2)
    print(svar_b_3)
    print(svar_b_4)

# c) Minste samlede areal til platene når kassen er 80 cm^3
if _blokk == 1:

    # Minst arealsteg-for-steg
    if _blokk == 0:
        # 1. Når vi ser på uttrykkene for A og V, så ser
        #    vi at dette er et likningssett med to ukjente, s og h
        # 2. På samme måte som i oppg. b), kan vi derfor
        #    først bruke V og finne et uttrykk for h mhp. s
        #    (volumet V er kjent, 80)
        # 4. Dette uttrykket for h kan vi deretter sette inn i A,
        #    som så blir uttrykt kun med s
        # 5. A kan derfor deriveres mhp. s
        #    Detter løser vi likningen dA = 0 på samme måte som
        #    i oppg. b) for å finne den verdien av s som gir det laveste arealet
        # 6. Når vi har funnet denne verdien for s, så kan vi regne ut hvor
        #    stort dette arealet er ved å sette den inn i A
        pass

    # Løser likningen 80 = s**2 * h for h
    h_uttrykk = superløs(variabel = h,
                         vs       = V_80,
                         hs       = V,
                         rund     = -1,
                         debug    = -1) # h = 80/s**2

    # Setter h = 80/s**2 inn i A og definerer det nye uttrykket som A_s 
    A_s = A.subs(h, h_uttrykk)

    # Deriverer A mhp. s og får dA = 2*s - 320/s**2
    dA = A_s.diff()

    # Løser likningen dA = 0 
    s_bunn = superløs(variabel = s,
                      vs       = dA,
                      hs       = 0,
                      rund     = -1,
                      debug    = -1) # s = 2*20**(1/3)

    # Setter s = 2*20**(1/3) inn i A_s og definerer det nye uttrykket som A_min 
    A_min = A_s.subs(s, s_bunn)

    # Runder av svaret
    A_min_rund = round(A_min, 2)

    # Print svar-setninger
    svar_c_0 = f'Oppg c)'
    svar_c_1 = f'- Fordi a-koeffisienten til A er positiv, så "smiler" grafen'
    svar_c_2 = f'- Dette betyr at vi har et bunnpunkt (grunnen til at vi valgte variabelnavnet s_bunn)'
    svar_c_3 = f'- I dette bunnpunktet er det minste areaelet A = {A_min_rund} dm^2'
    print("")
    print(svar_c_0)
    print(svar_c_1)
    print(svar_c_2)
    print(svar_c_3)



import matematikk as mt

def sannsynlighet_x_fra_mu_sd(
        mu                      = float(),
        sd                      = float(),
        x                       = float(),
        x_ulikhetstegn          = ">",
        X                       = float(),
        X_ulikhetstegn          = ">",
        k                       = float(),
        k_ulikhetstegn          = ">",
        gunstig                 = float(),
        gunstig_ulikhetstegn    = ">",
        sim                     = int(),
        is_prosent              = -1,
        desimal                 = -1,
        is_print                = -1,
        print_txt               = str()
    ):

    # Alias-argumenter > Gunstig
    _gunstig = float()
    if x        != float(): _gunstig = x
    if X        != float(): _gunstig = X
    if k        != float(): _gunstig = k
    if gunstig  != float(): _gunstig = gunstig

    # Alias-argumenter > Gunstig
    _x_ulikhetstegn = ">"
    if x_ulikhetstegn       != ">": _x_ulikhetstegn = x_ulikhetstegn
    if X_ulikhetstegn       != ">": _x_ulikhetstegn = X_ulikhetstegn
    if k_ulikhetstegn       != ">": _x_ulikhetstegn = k_ulikhetstegn
    if gunstig_ulikhetstegn != ">": _x_ulikhetstegn = gunstig_ulikhetstegn

    # Interne variabeler
    _rand           = float()   # Generer tilfeldig verdi fra standard normalfordeling
    _gunstig_ls     = list()    # Lag liste for gunstige utfall
    _gunstig_ant    = list()    # Antall gunstige: Tell opp el i _gunstig_ls med len(_gunstig_ls)

    # ...
    for i in range(sim):

        # ...
        _rand = mt.random.normal(mu, sd) # Alternativ: np.random.normal(mu, sd) med import numpy as np

        # ...
        if _x_ulikhetstegn == ">":
            if _rand > _gunstig:
                _gunstig_ls.append(_rand)
        if _x_ulikhetstegn == "<":
            if _rand < _gunstig:
                _gunstig_ls.append(_rand)

    # Antall gunstige el i listen
    _gunstig_ant = len(_gunstig_ls)

    # Sannsynlighet er definert som P = g / m
    sannsyn = (_gunstig_ant / sim)

    # Hvis svaret skal konverteres til prosent
    if is_prosent   != -1: sannsyn *= 100

    # Hvis svaret skal avrundes
    if desimal      != -1: sannsyn = round(sannsyn, desimal)

    # Hvis svaret skal avrundes
    if is_print     != -1: print(f"{print_txt}{sannsyn} %")

    return sannsyn



def sannsynlighet_x_mindre_enn_fra_mu_sd(
        mu                      = float(),
        sd                      = float(),
        k_start                 = 0,
        k_ulikhetstegn          = ">",
        k_txt                   = "k",
        sim                     = int(),
        sannsyn_grense          = float(),
        sannsyn_grense_desimal  = -1,
        is_prosent              = -1,
        desimal                 = -1,
        is_debug                = -1,
        debug_txt_new_line      = 1,
        debug_txt_k             = "k              :: ",
        debug_txt_sannsyn       = "Sannsynlighhet :: ",
        is_print                = -1,
        print_x_txt             = "",
        is_print_tolkning       = 1,
        print_tolkning_txt      = "Praktisk tolkning:",
    ):

    # Bestem k slikat P(X <= k) = sannsyn_grense.
    # Gi en praktisk tolkning av svaret.

    # Oppg b)
    _sannsyn            = 0         # Sannsynligheten starter på 0 i while-løkken
    _k                  = k_start   # Starter på 0 og øker med k_ink
    _k_ink              = 1         # Steglengde som vi øker bremselengden med

    while _sannsyn < sannsyn_grense:

        # Debug
        if is_debug != -1:
            if debug_txt_new_line == 1: print("")
            print(f"{debug_txt_k}{_k}")
            print(f"{debug_txt_sannsyn}{_sannsyn}")

        _sannsyn = mt.sannsynlighet_x_fra_mu_sd( # sannsynlighet_x_fra_mu_sd_EX
            mu              = mu,
            sd              = sd,
            k               = _k,
            k_ulikhetstegn  = k_ulikhetstegn,
            sim             = sim,
            is_prosent      = is_prosent,
            desimal         = desimal,
            is_print        = is_print,
            print_txt       = print_x_txt,
        )

        _k += _k_ink

    # Avrunding
    if sannsyn_grense_desimal != -1: _sannsyn = round(_sannsyn, sannsyn_grense_desimal)

    # Tolkning
    if is_print_tolkning == 1:

        print("")
        if print_tolkning_txt == "Praktisk tolkning:": print(print_tolkning_txt)

        print(f"{_sannsyn} sannsynlighet for at {k_txt} er {k_ulikhetstegn} {_k} m")



def sannsynlighet_gjennomsnitt_x_fra_mu_sd(
    mu                  = float(),
    sd                  = float(),
    x                   = float(),
    x_enhet             = str(),
    x_ulikhet           = ">",
    x_gjennomsnitt_ant  = int(),
    x_desimal           = -1,
    sim                 = int(),
    is_print            = -1
):

    # Nullstilling
    _x_enhet        = x_enhet
    _rand_ls        = list()
    _g_ls           = list()
    _rand_snitt     = float()
    _snitt_sannsyn  = float()

    # Mellomrom
    if _x_enhet != "": _x_enhet = " " + _x_enhet # Ikke def: _x_enhet =+ " "

    # Simulering f.eks. 100 000 ganger
    for _ in range(sim):

        # Husk å resette listen med tilfeldige verdier
        _rand_ls = []

        for _ in range(x_gjennomsnitt_ant):
            _rand = mt.random.normal(mu, sd) # np.random.normal(mu, sd) kan også brukes med import numpy as np
            _rand_ls.append(_rand)

        # Tag snittet av de 15 målingene
        _rand_snitt = mt.mean(_rand_ls)

        # Hvis snittet er mindre enn 84 telles denne som en gunstig
        if x_ulikhet == "<":
            if _rand_snitt < x: _g_ls.append(1)
        if x_ulikhet == ">":
            if _rand_snitt > x: _g_ls.append(1)

    # Sannsynlighet er definert som P = g / m
    _snitt_sannsyn = len(_g_ls) / sim

    # Hvis avrunding
    if x_desimal != -1: _snitt_sannsyn = round(_snitt_sannsyn, x_desimal)

    # Print
    if is_print == -1:
        print(f"Sannsynligheten for at gjennomsnittet av {x_gjennomsnitt_ant} ", )
        print(f"målinger er større en {x}{_x_enhet} er: {_snitt_sannsyn}")

    return _snitt_sannsyn



##########################################
# sannsynlighet_x_fra_mu_sd()
##########################################

# Alias > 1 > sannsynlighet_x_fra_mu_sd
sannsynlighet_X_fra_mu_sd       = sannsynlighet_x_fra_mu_sd
sannsynlighet_k_fra_mu_sd       = sannsynlighet_x_fra_mu_sd
sannsynlighet_gunstig_fra_mu_sd = sannsynlighet_x_fra_mu_sd

# Alias > 2 > sannsynlighet_x_fra_mu_sd > ...


##########################################
# sannsynlighet_x_mindre_enn_fra_mu_sd()
##########################################

# Alias > 1 > sannsynlighet_x_mindre_enn_fra_mu_sd > ...
sannsynlighet_x_storre_enn_fra_mu_sd = sannsynlighet_x_mindre_enn_fra_mu_sd

# Alias > 2 > sannsynlighet_x_mindre_enn_fra_mu_sd > ...


##########################################
# sannsynlighet_gjennomsnitt_x_fra_mu_sd()
##########################################

# Alias > 1 > sannsynlighet_x_mindre_enn_fra_mu_sd > ...
sannsynlighet_gjennomsnitt_x_mindre_fra_mu_sd = sannsynlighet_gjennomsnitt_x_fra_mu_sd

# Alias > 2 > sannsynlighet_x_mindre_enn_fra_mu_sd > ...




#from _defs.def_bld import (
#    bld
#)
#from _defs.def_col import (
#    col_
#)
#from _konst.prim_col0 import Col0
#_col0 = Col0()

import matematikk as mt # Eksamen
from colorama import Fore, Style

def økonomi(
    oppg_typ                        = "",
    I_uttrykk                       = mt.Symbol(""),
    pris_uttrykk_hs_desimal         = -1,
    pris_uttrykk_hs_rund            = -1,
    pris_uttrykk_hs                 = mt.Symbol(""),
    pris_desimal                    = -1,
    pris_rund                       = -1,
    pris                            = mt.Symbol(""),
    p_desimal                       = -1,
    p_rund                          = -1,
    p                               = mt.Symbol(""),
    ):

    # Oppg
    _oppg_typ   = ""
    _tit        = ""

    # Økonomi
    if oppg_typ                     == "":

        _oppg_typ   = "Økonomi"
        _tit        = "Økonomi"

    # Eksamen
    if oppg_typ                     != "":

        _oppg_typ   = "Eksamen" # oppg_typ
        _tit        = "Eksamen" # oppg_typ

    # Pris
    _p = mt.Symbol("")
    if p                            != mt.Symbol(""): _p = p
    if pris                         != mt.Symbol(""): _p = p
    if pris_uttrykk_hs              != mt.Symbol(""): _p = p

    # Pris avrunding
    _p_desimal = 0
    if pris_uttrykk_hs_desimal      != -1: _p_desimal = pris_uttrykk_hs_desimal
    if pris_uttrykk_hs_rund         != -1: _p_desimal = pris_uttrykk_hs_desimal
    if pris_desimal                 != -1: _p_desimal = pris_desimal
    if pris_rund                    != -1: _p_desimal = pris_rund
    if p_desimal                    != -1: _p_desimal = p_desimal
    if p_rund                       != -1: _p_desimal = p_rund

    # Inntekts-funksjon
    _I = mt.Symbol("")
    if I_uttrykk                    != mt.Symbol(""): _I = I_uttrykk
    if I_uttrykk                    == mt.Symbol(""): _I = _p * x

    # Variabler
    _pr_ls                          = list()
    _p_fn                           = "-"
    _I_fn                           = "-"
    _inntekt_max_enheter            = "-"
    _inntekt_max_pris               = "-"

    # Pris-funksjon
    if _p == mt.Symbol(""): pass
    if _p != mt.Symbol(""):

        _p_fn = f"p(x) = {_p}"

        # Antall enheter (etterspørsel) og pris per enhet når inntekten er størst
        enhet_og_pris_ls = mt.enhet_og_pris_fra_inntekt_max(pris_uttrykk_hs = p,
                                                            pris_desimal    = _p_desimal,
                                                            enhet_vari      = mt.Symbol("x"),
                                                            enhet_desimal   = None,
                                                            enhet_debug     = -1)

    # Inntekst-funksjon
    if _I == mt.Symbol(""): _I = _p * x
    if _I_fn == "-":

        _I_fn = "I(x)" + " " + "=" + " " + str(_I)

    # Størst inntekt > Antall enheter
    if _p_fn == "-": pass
    if _p_fn != "-":

        _inntekt_max_enheter = "x = " f"{enhet_og_pris_ls[0]}"

    # Størst inntekt
    if _p_fn == "-": pass
    if _p_fn != "-":

        _inntekt_max_pris_val = round(float(enhet_og_pris_ls[1]), 2)
        _inntekt_max_pris = "p(x) = " + f"{_inntekt_max_pris_val}" + " " + "kr"

    _colors = 1
    _is_colors = 0
    # 1: Light bvit
    # 2: Blå
    # 3: Rød
    _col_tit        = 1
    _col_ramme      = 3
    _col_uttrykk    = 2
    #def cc(txt, typ = 0):
    #    if _colors == 0: return txt
    #    if _colors == 1:
    #        if typ == 0: return col_(txt, _col0._green)
    #        if typ == 1: return col_(txt, _col0._green)
    #        if typ == 2: return col_(txt, _col0._magenta)
    #        if typ == 3: return col_(txt, _col0._red)
    #        if typ == 4: return col_(txt, _col0._blue)

    if _oppg_typ == "Økonomi":

        _pr_ls.append(f"╔══════════════════════════════════════") # ┌
        _pr_ls.append(f"║ {_tit}                            ") # │
        _pr_ls.append(f"╠───┬──────────────────────────────────")
        _pr_ls.append(f"║   │ Pris-funksjon                    ") # │
        _pr_ls.append(f"║   └───┬─────────┬────────────────────")
        _pr_ls.append(f"║       │ Uttrykk │ {_p_fn} ")
        _pr_ls.append(f"║   ┌───┴─────────┴────────────────────")
        _pr_ls.append(f"║   │ Inntekt-funksjon                 ")
        _pr_ls.append(f"║   └───┬─────────┬────────────────────")
        _pr_ls.append(f"║       │ Uttrykk │ {_I_fn} ")
        _pr_ls.append(f"║       ├─────────┴──────┬─────────┬")
        _pr_ls.append(f"║       │ Størst inntekt │ Enheter │ {_inntekt_max_enheter} ")
        _pr_ls.append(f"║       └────────────────┼─────────┼")
        _pr_ls.append(f"║                        │ Pris    │ {_inntekt_max_pris} ")
        _pr_ls.append(f"╚════════════════════════╩═════════╩") #  └ ╩ ┴ ┴

    # Eksamen
    if _oppg_typ == "Eksamen":

        _pr_ls.append(f"╔══════════════════════════════════════") # ┌
        _pr_ls.append(f"║ {_tit}                            ") # │
        _pr_ls.append(f"╠───┬──────────────────────────────────") # ├
        _pr_ls.append(f"║   │ Økonomi                    ") # │
        _pr_ls.append(f"║   └───┬─────────────────────────────")
        _pr_ls.append(f"║       │ Pris-funksjon                    ") # │
        _pr_ls.append(f"║       └───┬─────────┬────────────────────")
        _pr_ls.append(f"║           │ Uttrykk │ {_p_fn} ")
        _pr_ls.append(f"║       ┌───┴─────────┴────────────────────")
        _pr_ls.append(f"║       │ Inntekt-funksjon                 ")
        _pr_ls.append(f"║       └───┬─────────┬────────────────────")
        _pr_ls.append(f"║           │ Uttrykk │ {_I_fn} ")
        _pr_ls.append(f"║           ├─────────┴──────┬─────────┬")
        _pr_ls.append(f"║           │ Størst inntekt │ Enheter │ {_inntekt_max_enheter} ")
        _pr_ls.append(f"║           └────────────────┼─────────┼")
        _pr_ls.append(f"║                            │ Pris    │ {_inntekt_max_pris} ")
        _pr_ls.append(f"╚════════════════════════════╩═════════╩") #  └ ╩ ┴ ┴


    _cnt    = 0
    _i_max  = list()

    # Finn lengste lengde
    for i in range(len(_pr_ls)):
        if len(_pr_ls[i]) > _cnt: _cnt = len(_pr_ls[i])

    # Lagre index med max lengde
    for i in range(len(_pr_ls)):
        if len(_pr_ls[i]) == _cnt: _i_max.append(i)

    # Lag streker / space
    for i in range(len(_pr_ls)):

        if len(_pr_ls[i]) < _cnt:
            _diff = _cnt - len(_pr_ls[i])
            _strek = ""
            _STREK = ""
            _space = ""
            for _ in range(_diff):
                _strek += r"─"
                _STREK += r"═"
                _space += f" "
 
        # Max lengde
        if i in _i_max: _pr_ls[i] = _pr_ls[i] + "║" # + "│"

        # Vanlig lengde
        else:

            if _oppg_typ == "Økonomi":

                if i == 0:      _pr_ls[i] = _pr_ls[i] + _STREK + "╗" # + "┐"
                if i == 1:      _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 2:      _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 3:      _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 4:      _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 5:      _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 6:      _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 7:      _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 8:      _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 9:      _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 10:     _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 11:     _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 12:     _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 13:     _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 14:     _pr_ls[i] = _pr_ls[i] + _STREK + "╝" # + "┘" # Siste linje

            if _oppg_typ == "Eksamen":

                if i == 0:      _pr_ls[i] = _pr_ls[i] + _STREK + "╗" # + "┐"
                if i == 1:      _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 2:      _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 3:      _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 4:      _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 5:      _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 6:      _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 7:      _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 8:      _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 9:      _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 10:     _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 11:     _pr_ls[i] = _pr_ls[i] + _space + "║" # + "│"
                if i == 12:     _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┤"
                if i == 13:     _pr_ls[i] = _pr_ls[i] + _space + "║" # + "┤"
                if i == 14:     _pr_ls[i] = _pr_ls[i] + _strek + "╣" # + "┘"
                if i == 15:     _pr_ls[i] = _pr_ls[i] + _strek + "║" # + "┘"
                if i == 16:     _pr_ls[i] = _pr_ls[i] + _STREK + "╝" # + "┘" # Siste linje

    _debug_col = 3
    if _debug_col == 0:
        for pr in _pr_ls: print(pr)
    if _debug_col == 1:
        for pr in _pr_ls: print(f"{pr} {len(pr)}")

    _col_ls = list()
    if _debug_col == 3:
        for i in range(len(_pr_ls)):

            from colorama import Fore, Style

            if _is_colors == 0: _col_ls.append(f"{_pr_ls[i]}")
            if _is_colors == 1: _col_ls.append(f"{Fore.RED}{_pr_ls[i]}")

            def colo(typ):

                if _is_colors == 0: return ""

                if _is_colors == 1:

                    if typ == "r":

                        if _col_ramme   == 2: return Fore.BLUE
                        if _col_ramme   == 3: return Fore.RED

                    if typ == "t":

                        if _col_tit     == 1: return Fore.LIGHTWHITE_EX

                    if typ == "u":

                        if _col_uttrykk == 2: return Fore.BLUE

            # Ramme > Single
            _col_ls[i] = str(_col_ls[i]).replace('─', f'{colo("r")}─{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('┌', f'{colo("r")}┌{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('│', f'{colo("r")}│{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('┬', f'{colo("r")}┬{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('└', f'{colo("r")}└{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('┴', f'{colo("r")}┴{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('┤', f'{colo("r")}┤{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('┐', f'{colo("r")}┐{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('├', f'{colo("r")}├{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('┼', f'{colo("r")}┼{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('┘', f'{colo("r")}┘{colo("u")}') # {Style.RESET_ALL}

            # Ramme > Dobbel
            _col_ls[i] = str(_col_ls[i]).replace('═', f'{colo("r")}═{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('╩', f'{colo("r")}╩{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('╗', f'{colo("r")}╗{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('║', f'{colo("r")}║{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('╣', f'{colo("r")}╣{colo("u")}') # {Style.RESET_ALL}
            _col_ls[i] = str(_col_ls[i]).replace('╝', f'{colo("r")}╝{colo("u")}') # {Style.RESET_ALL}

            # Tittel
            _col_ls[i] = str(_col_ls[i]).replace("Eksamen",             f'{colo("t")}Eksamen') # {Style.RESET_ALL} Etter alle ord
            _col_ls[i] = str(_col_ls[i]).replace("Alle svar",           f'{colo("t")}Alle svar')
            _col_ls[i] = str(_col_ls[i]).replace("Økonomi",             f'{colo("t")}Økonomi')
            _col_ls[i] = str(_col_ls[i]).replace("Pris-funksjon",       f'{colo("t")}Pris-funksjon')
            _col_ls[i] = str(_col_ls[i]).replace("Uttrykk",             f'{colo("t")}Uttrykk')
            _col_ls[i] = str(_col_ls[i]).replace("Inntekt-funksjon",    f'{colo("t")}Inntekt-funksjon')
            _col_ls[i] = str(_col_ls[i]).replace("Størst inntekt",      f'{colo("t")}Størst inntekt')
            _col_ls[i] = str(_col_ls[i]).replace("Enheter",             f'{colo("t")}Enheter')
            _col_ls[i] = str(_col_ls[i]).replace("Pris",                f'{colo("t")}Pris')

    # bld(cmd="clear") # Sjekk Mac / Kanskje lin
    for pr in _col_ls: print(pr)

# Exe
x = mt.Symbol("x")
_exe_okonomi = 0
if _exe_okonomi == 1:

    alle_svar = økonomi(
        p = 79 - 12.2 * mt.ln(x)
    )

### _exe_eksamen = 0
### if _exe_eksamen == 1:

def eksamen(p, oppg_typ = "eksamen"):
    def _økonomi(p, oppg_typ):
        alle_svar = økonomi(
            p = 79 - 12.2 * mt.ln(x),
            oppg_typ = oppg_typ
        )
    _økonomi(p = p, oppg_typ = oppg_typ)

### alle_svar = eksamen(
###     p = 79 - 12.2 * mt.ln(x)
### )



# Alias > 1
#derivert                        = diff
#momentan_vekst                  = diff
#momentan_vekstfart              = diff

# Alias > 2 > ...



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (NDLA)
# Oppgave 4 a) Aritmetisk- og geometrisk rekke - Ukentlig beløp de 4 første ukene (alternativ versjon) 

an = 100            # setter startverdi for ukelønna
a = [] 				# lager tom liste for ukelønn tilbud 1
bn = 100            # setter startverdi for ukelønna
b = [] 				# lager tom liste for ukelønn tilbud 2

for i in range(4):	# for-løkke som kjører 4 ganger
	a.append(an) 	# legger til gjeldende ukelønn til lista a
	an = an + 10 	# regner ut ukelønn for neste uke
	b.append(bn) 	# legger til gjeldende ukelønn til lista b
	bn = bn*1.05    # regner ut ukelønn for neste uke

print("Tilbud 1:")
print(a)            # skriver ut lista med ukelønnene
print("Tilbud 2:")
print(b)            # skriver ut lista med ukelønnene



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 c) og d) Etterspørsel 
# - Løser oppgaven med CAS i Python
# - Kopier denne kommandoen i terminalen for å importere sympy: 
#   $ pip install sympy

import sympy as sp

_blokk = 1
if _blokk == 1:

    # Variabler
    x                   = sp.Symbol("x")        # Etterspørsel (antall produserte enheter)
    p                   = sp.Symbol("p")        # Pris per enhet
    I                   = sp.Symbol("I")        # Inntekt
    K                   = sp.Symbol("K")        # Kostnad
    svar_c_ls           = list()                # Svar-setninger
    svar_d_ls           = list()                # Svar-setninger
    desimal_p           = 1                     # Avrunding
    desimal_x           = None                  # Avrunding

# Felles: Oppg c) og d)
if _blokk == 1:

    # Definerer uttrykket for pris-funksjonen, p(x)
    p = 79 - 12.2 * sp.ln(x)

    # Definerer uttrykket for inntekts-funksjonen, I(x)
    I = x * p

    # Deriverer I mhp. x og får dI = 66.8 - 12.2*log(x)
    dI = sp.diff(I, x) # I'(x)

# Oppg c)
if _blokk == 1:

    # Lager likningen dI = 0 med sp.Eq(vs, hs)
    likning = sp.Eq(dI, 0)

    # Løser likningen for x med solveset() og får løsnings-settet {238.748294544347}
    x_inntekt_max_set = sp.solveset(likning, x, domain=sp.Reals) # Løsnings-domene i ℝ (Reals)

    # Henter elementet i fra løsnings-settet, 238.748294544347
    x_inntekt_max = x_inntekt_max_set.args[0]

    # Runder av 238.748294544347 -> 239
    x_inntekt_max = round(x_inntekt_max, None)

    # Setter x_inntekt_max = 239 inn i p og definerer det nye uttrykket som p_inntekt_max
    p_inntekt_max = p.subs(x, x_inntekt_max)

    # Henter verdien til p_inntekt_max
    p_inntekt_max_val = p_inntekt_max.evalf()

    # Runder av 12.1871446664356 -> 12.20
    p_inntekt_max_val = round(p_inntekt_max_val, desimal_p)

    # Svar-setninger
    svar_c_ls.append(f"")
    svar_c_ls.append(f"Oppg c)")
    svar_c_ls.append(f"")
    svar_c_ls.append(f"- Inntekten er størst når etterspørselen er ca. {x_inntekt_max} enheter")
    svar_c_ls.append(f"- Dette gir en pris på ca. {p_inntekt_max_val} kr")

    # Print svar-setninger
    for svar in svar_c_ls: print(svar)

# Oppg d)
if _blokk == 1:

    # Definerer uttrykket for kostnads-funksjonen, K(x)
    K = 0.021 * x**2 + 10 * x + 910

    # Deriverer K mhp. x og får dK = 0.042*x + 10
    dK = sp.diff(K, x) # K'(x)

    # Lager likningen dI = dK mhp. x med sp.Eq(vs, hs)
    likning = sp.Eq(dI, dK)

    # Løser likningen med nsolve() og får 79.8935006531202
    x_opt = sp.nsolve(likning, x, 1)

    # Runder av f.eks. 1234.56789 -> 1234.6
    x_opt = round(x_opt, desimal_x)

    # Svar-setninger
    svar_d_ls.append(f"")
    svar_d_ls.append(f"Oppg d)")
    svar_d_ls.append(f"")
    svar_d_ls.append(f"- Grense-inntekten er lik grense-kostnaden ved {x_opt} enheter")
    svar_d_ls.append(f"- Dette betyr at {x_opt} enheter gir størst overskudd fordi:")
    svar_d_ls.append(f"     O`(x) = 0, max overskudd")
    svar_d_ls.append(f"     O`(x) = I`(x) - K`(x), definisjonen av overskudd")
    svar_d_ls.append(f"     0 = I`(x) - K`(x)")
    svar_d_ls.append(f"     K`(x) = I`(x)")

    # Print svar-setninger
    for svar in svar_d_ls: print(svar)



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Torodd)
# Oppgave 2 b) Venstrehendte - P("Minst 3 venstrehendte gutter") > 0.2 

import numpy as np

# definerer tilfeldighetsgeneratoren
rng = np.random.default_rng()

def minst_tre_av(n):
    # tal simuleringar
    N = 1000

    # simulerer uttrekk av n gutar
    simulert = rng.binomial(n = n, p = 0.1, size = N)

    # tel opp gunstige utfall og finn sannsynet
    gunstige = sum(simulert >= 3)
    sannsyn = gunstige / N

    return sannsyn

n_gutar = 3

while minst_tre_av(n_gutar) < 0.20:
    n_gutar += 1

print(n_gutar)



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 c) og d) Alle svar - økonmi - DEMO
# - Løser oppgaven med CAS i Python
# - Kopier denne kommandoen i terminalen for å importere matematikk:
#   $ pip install matematikk

import matematikk as mt
x = mt.Symbol("x")
alle_svar = mt.økonomi(
    p = 79 - 12.2 * mt.ln(x)
)




# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (NDLA)
# Oppgave 3 b) Coca-Cola vs. Pepsi-Cola - Hypotese-test 

from scipy.stats import binom
import numpy as np

n = 10      # antall forsøk
p = 0.5     # sannsynligheten
X = []      # ei liste for de ulike verdiene X kan ha

for i in range(n+1):
    X.append(i) # legger inn alle de mulige verdiene til X
rettesvar = binom.pmf(X,n,p) # lager en array med sannsynlighetsfordelingen
merenna = np.cumsum(rettesvar) #finner de kumulative sannsynlighetene
a = 8 #antall rette

# sjekker om summen av sannsynlighetene for X>a er
# større eller mindre enn 0.05
if 1 - merenna[a-1] < 0.05:
    print("Det er sannsynlig at Marte kan gjenkjenne de to colatype.")
else:
    print("Det er ikke sannsynlig at Marte kan gjenkjenne de to colatype.")



# 🚀 programmering.no | 🤓 matematikk.as
# - Se om listen er sortert i stigende eller synkende rekkefølge (monotonisk)
# - Kort versjon med all()

# Input
liste = input("Liste: ") # Husk [ ], f.eks. [1, 2, 3, 4]

# Enkel funksjon som klipper opp inp-str til liste med int-el
def str_til_int_liste(str_liste = list()):

    int_liste = list()
    el = str()
    for i in range(len(str_liste)):
        if str_liste[i] == " ": pass
        elif str_liste[i] == "[": pass
        elif str_liste[i] == "]":
            int_liste.append(int(el))
            el = ""
        elif str_liste[i] == ",":
            int_liste.append(int(el))
            el = ""
        else: el += str_liste[i]
    return int_liste

# Funksjon som ser om listen er monotonisk eller ikke
def er_liste_monotonisk(liste):

	# Returner True hvis listen er monotonisk og False hvis ikke
	return (all(liste[i] <= liste[i + 1] for i in range(len(liste) - 1)) or
			all(liste[i] >= liste[i + 1] for i in range(len(liste) - 1)))

# Input-string endres til liste med int-el
liste = str_til_int_liste(liste)

# Monotonisk eller ikke
er_monotonisk = er_liste_monotonisk(liste)

# Liste
if er_monotonisk == True:
	print(f"Er listen sortert? Ja")
if er_monotonisk == False:
	print(f"Er listen sortert? Nei")



# 🚀 programmering.no | 🤓 matematikk.as
# - Teller hvor mange utvalgte elementer det er i en liste

# Verdier
liste    = ["A", "B", "B", "C", "F"]
karakter = "B"

# Antall
antall = liste.count("B")

# Print
print(f"Liste: {liste}")
if antall > 0:
    print(f"{antall} elev(er) fikk karakter {karakter}")
else:
	print("Ingen elever fikk karakter {karakter}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Regn ut arealet av en sirkel

# Verdier
pi = 3.14
r  = 1.5

# Areal-funksjon for sirkel
def sirkel_areal(r):

    # Formel for arealet av sirkel
    areal = pi * float(r)**2

    return areal

# Areal
areal = sirkel_areal(r)

# Print
print(f"Arealet er:: {areal}")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (NDLA)
# Oppgave 3 c) Coca-Cola vs. Pepsi-Cola - 20 glass 

from scipy.stats import binom
import numpy as np

n = 30      # antall forsøk
p = 0.5     # sannsynligheten
X = []      # ei liste for de ulike verdiene X kan ha

for i in range(n+1):
    X.append(i) # legger inn alle de mulige verdiene til X
rettesvar = binom.pmf(X,n,p) # lager en array med sannsynlighetsfordelingen
merenna = np.cumsum(rettesvar) #finner de kumulative sannsynlighetene
a = 15 #antall rette
sannsynlighet = 1 #starter løkka med sannsynlighet 1

while sannsynlighet > 0.05:
    sannsynlighet = 1 - merenna[a]
    a = a + 1

print(f"Marte må svare rett minst {a} ganger for at det skal være sannsynlig at huns smaker forskjell")
